#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QTextCodec>
#include <QDir>
#include <QSettings>

#include <QMessageBox>
#include <QDebug>

#include "logstartdialog.h"


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    // set Codec
    QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF-8")); // source codec is UTF-8

    QCoreApplication::setApplicationName("QtUSBRH-Logger-win32");
    QCoreApplication::setApplicationVersion("0.1");

    IniFilepath = QCoreApplication::applicationDirPath() + QDir::separator() + QCoreApplication::applicationName() + ".ini";
    // Load Setting
    LoadSaveSetting(IniFilepath, false);

    ui->lcdTempr->display(0.0);
    ui->lcdHumid->display(0.0);

    ui->label_LogLapsedTime->setText(""); // "Lapsed Time: "
//    ui->label_LogLapsedTime->hide(); // hidden label_LogLapsedTime


    logging = false;

    tmpr_max = -1000.0;
    tmpr_min = 1000.0;
    time_max.setTime_t(0);
    time_min.setTime_t(0);

    // Init & Start DAQ Thread
    daqTh = new DAQThread(this);
    // connect signals
    connect(this, SIGNAL(LogStart(QString)),
            daqTh, SIGNAL(LogStart(QString)));
    connect(this, SIGNAL(LogStop()),
            daqTh, SIGNAL(LogStop()));
    connect(daqTh, SIGNAL(LogStarted()),
            this, SLOT(slotLogStarted()));
    connect(daqTh, SIGNAL(LogEnded(int)),
            this, SLOT(slotLogEnded(int)));
    connect(daqTh, SIGNAL(GetData(QDateTime, double, double)),
            this, SLOT(slotGetData(QDateTime, double, double)));
    connect(daqTh, SIGNAL(ChangeLogFilename(QString)),
            this, SLOT(slotChangeLogFilename(QString)));
    // DaqTh start
    daqTh->start();

}

MainWindow::~MainWindow()
{
    // Save Setting
    LoadSaveSetting(IniFilepath, true);

    // stop daqTh
    daqTh->stop();
    daqTh->wait();
    delete daqTh;

    delete ui;
}

void MainWindow::LoadSaveSetting(QString filepath, bool store)
{
    QSettings settings(filepath, QSettings::IniFormat);
    settings.setIniCodec("UTF-8");
    if (store) { // Store(Save)
        settings.setValue("SETTING/SAVEPATH", logpath);
    } else { // Load
        logpath = settings.value("SETTING/SAVEPATH", QString("")).toString();
    }
}


void MainWindow::slotGetData(QDateTime timestamp, double tmpr, double humid)
{
    // qDebug() << "timestamp=" << timestamp << ", Tempr=" << tmpr << ", humid=" << humid;
    ui->lcdTempr->display(tmpr);
    ui->lcdHumid->display(humid);

    if (tmpr > tmpr_max ) {
        tmpr_max = tmpr;
        time_max = timestamp;
    }
    if (tmpr < tmpr_min ) {
        tmpr_min = tmpr;
        time_min = timestamp;
    }
    ui->label_Max->setText(QString(tr("Max: %1 'C at ")).arg(tmpr_max) + time_max.toString("yyyy/MM/dd hh:mm:ss"));
    ui->label_Min->setText(QString(tr("Min: %1 'C at ")).arg(tmpr_min) + time_min.toString("yyyy/MM/dd hh:mm:ss"));

    if (logging) {
        int lapsed_time = (QDateTime::currentMSecsSinceEpoch() - logstart_time.toMSecsSinceEpoch()) / 1000;
        if (lapsed_time < 60) { // 1 minute
            ui->label_LogLapsedTime->setText(QString(tr("Lapsed Time: %1 sec")).arg(lapsed_time));
        } else if (lapsed_time < 3600) { // 1 hour
            ui->label_LogLapsedTime->setText(QString(tr("Lapsed Time: %1 min")).arg(lapsed_time/60));
        } else if (lapsed_time < 86400) { // 1 day
            ui->label_LogLapsedTime->setText(QString(tr("Lapsed Time: %1 hour")).arg(lapsed_time/3600));
        } else { // over 1 day
            ui->label_LogLapsedTime->setText(QString(tr("Lapsed Time: %1 day")).arg(lapsed_time/86400));
        }
    }
}
void MainWindow::slotChangeLogFilename(QString filename)
{
    ui->statusBar->showMessage(QString(tr("logfile: %1")).arg(filename));
}

void MainWindow::slotLogStarted()
{
    // ui->label_LogState->setText("LogStarted: " + QDateTime::currentDateTime().toString("yyyy/MM/dd hh:mm:ss"));
    ;
}
void MainWindow::slotLogEnded(int status)
{
    logging = false;
    ui->pushButton_LogStart->setText(tr("Log Start"));

    ui->label_LogState->setText(tr("LogEnded: ") + QDateTime::currentDateTime().toString("yyyy/MM/dd hh:mm:ss"));
}


void MainWindow::on_actionExit_triggered()
{
    close();
}

void MainWindow::on_pushButton_LogStart_clicked()
{
    if (logging) {
        // now logging
        // Send LogStop
        emit LogStop();

        logging = false;
        ui->pushButton_LogStart->setText(tr("Log Start"));
        ui->label_LogState->setText(tr("LogEnded: ") + QDateTime::currentDateTime().toString("yyyy/MM/dd hh:mm:ss"));
        ui->label_LogLapsedTime->setText("");
    } else {
        // now idle..
        LogStartDialog *dlg = new LogStartDialog(logpath, this);
        int i = dlg->exec();
        if (i == QDialog::Accepted) {
            qDebug() << "Ok";
            logpath = dlg->getLogSaveFolderPath();
            qDebug() << "logpath is " << logpath;

            logging = true;
            ui->pushButton_LogStart->setText(tr("Log Stop"));
            logstart_time = QDateTime::currentDateTime();
            ui->label_LogState->setText(tr("LogStarted: ") + logstart_time.toString("yyyy/MM/dd hh:mm:ss"));

            // reset Max/Min
            tmpr_max = -1000.0;
            tmpr_min = 1000.0;
            time_max.setTime_t(0);
            time_min.setTime_t(0);

            // Send LogStart
            emit LogStart(logpath);

        } else if (i == QDialog::Rejected) {
            qDebug() << "Cancel";
        } else {
            qDebug() << "Unkown";
        }
        delete dlg;
    }
}

void MainWindow::on_actionAbout_triggered()
{
    QMessageBox::about(this, "About", tr("QtUSBRH Logger for win32 v0.1"));
}

void MainWindow::on_actionAboutQt_triggered()
{
    QApplication::aboutQt();
}
